package com.sun.xml.internal.ws.api.config.management.policy;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import com.sun.xml.internal.ws.resources.ManagementMessages;
import com.sun.xml.internal.ws.policy.spi.AssertionCreationException;
import com.sun.xml.internal.ws.policy.PolicyAssertion;
import java.util.Collection;
import com.sun.xml.internal.ws.policy.sourcemodel.AssertionData;
import javax.xml.ws.WebServiceException;
import com.sun.xml.internal.ws.policy.PolicyMap;
import com.sun.xml.internal.ws.api.server.WSEndpoint;
import com.sun.istack.internal.logging.Logger;
import javax.xml.namespace.QName;

public class ManagedServiceAssertion extends ManagementAssertion
{
    public static final QName MANAGED_SERVICE_QNAME;
    private static final QName COMMUNICATION_SERVER_IMPLEMENTATIONS_PARAMETER_QNAME;
    private static final QName COMMUNICATION_SERVER_IMPLEMENTATION_PARAMETER_QNAME;
    private static final QName CONFIGURATOR_IMPLEMENTATION_PARAMETER_QNAME;
    private static final QName CONFIG_SAVER_IMPLEMENTATION_PARAMETER_QNAME;
    private static final QName CONFIG_READER_IMPLEMENTATION_PARAMETER_QNAME;
    private static final QName CLASS_NAME_ATTRIBUTE_QNAME;
    private static final QName ENDPOINT_DISPOSE_DELAY_ATTRIBUTE_QNAME;
    private static final Logger LOGGER;
    
    public static ManagedServiceAssertion getAssertion(WSEndpoint endpoint) throws WebServiceException {
        ManagedServiceAssertion.LOGGER.entering(new Object[] { endpoint });
        PolicyMap policyMap = endpoint.getPolicyMap();
        ManagedServiceAssertion assertion = (ManagedServiceAssertion)ManagementAssertion.getAssertion(ManagedServiceAssertion.MANAGED_SERVICE_QNAME, policyMap, endpoint.getServiceName(), endpoint.getPortName(), ManagedServiceAssertion.class);
        ManagedServiceAssertion.LOGGER.exiting(assertion);
        return assertion;
    }
    
    public ManagedServiceAssertion(AssertionData data, Collection<PolicyAssertion> assertionParameters) throws AssertionCreationException {
        super(ManagedServiceAssertion.MANAGED_SERVICE_QNAME, data, assertionParameters);
    }
    
    @Override
    public boolean isManagementEnabled() {
        String management = this.getAttributeValue(ManagedServiceAssertion.MANAGEMENT_ATTRIBUTE_QNAME);
        boolean result = true;
        if (management != null) {
            result = (management.trim().toLowerCase().equals("on") || Boolean.parseBoolean(management));
        }
        return result;
    }
    
    public long getEndpointDisposeDelay(long defaultDelay) throws WebServiceException {
        long result = defaultDelay;
        String delayText = this.getAttributeValue(ManagedServiceAssertion.ENDPOINT_DISPOSE_DELAY_ATTRIBUTE_QNAME);
        if (delayText != null) {
            try {
                result = Long.parseLong(delayText);
            }
            catch (NumberFormatException e) {
                throw (WebServiceException)ManagedServiceAssertion.LOGGER.logSevereException(new WebServiceException(ManagementMessages.WSM_1008_EXPECTED_INTEGER_DISPOSE_DELAY_VALUE(delayText), e));
            }
        }
        return result;
    }
    
    public Collection<ManagedServiceAssertion.ImplementationRecord> getCommunicationServerImplementations() {
        Collection<ManagedServiceAssertion.ImplementationRecord> result = new LinkedList();
        Iterator<PolicyAssertion> parameters = this.getParametersIterator();
        while (parameters.hasNext()) {
            PolicyAssertion parameter = (PolicyAssertion)parameters.next();
            if (ManagedServiceAssertion.COMMUNICATION_SERVER_IMPLEMENTATIONS_PARAMETER_QNAME.equals(parameter.getName())) {
                Iterator<PolicyAssertion> implementations = parameter.getParametersIterator();
                if (!implementations.hasNext()) {
                    throw (WebServiceException)ManagedServiceAssertion.LOGGER.logSevereException(new WebServiceException(ManagementMessages.WSM_1005_EXPECTED_COMMUNICATION_CHILD()));
                }
                while (implementations.hasNext()) {
                    PolicyAssertion implementation = (PolicyAssertion)implementations.next();
                    if (ManagedServiceAssertion.COMMUNICATION_SERVER_IMPLEMENTATION_PARAMETER_QNAME.equals(implementation.getName())) {
                        result.add(this.getImplementation(implementation));
                    }
                    else {
                        throw (WebServiceException)ManagedServiceAssertion.LOGGER.logSevereException(new WebServiceException(ManagementMessages.WSM_1004_EXPECTED_XML_TAG(ManagedServiceAssertion.COMMUNICATION_SERVER_IMPLEMENTATION_PARAMETER_QNAME, implementation.getName())));
                    }
                }
            }
            else {
                continue;
            }
        }
        return result;
    }
    
    public ManagedServiceAssertion.ImplementationRecord getConfiguratorImplementation() {
        return this.findImplementation(ManagedServiceAssertion.CONFIGURATOR_IMPLEMENTATION_PARAMETER_QNAME);
    }
    
    public ManagedServiceAssertion.ImplementationRecord getConfigSaverImplementation() {
        return this.findImplementation(ManagedServiceAssertion.CONFIG_SAVER_IMPLEMENTATION_PARAMETER_QNAME);
    }
    
    public ManagedServiceAssertion.ImplementationRecord getConfigReaderImplementation() {
        return this.findImplementation(ManagedServiceAssertion.CONFIG_READER_IMPLEMENTATION_PARAMETER_QNAME);
    }
    
    private ManagedServiceAssertion.ImplementationRecord findImplementation(QName implementationName) {
        Iterator<PolicyAssertion> parameters = this.getParametersIterator();
        while (parameters.hasNext()) {
            PolicyAssertion parameter = (PolicyAssertion)parameters.next();
            if (implementationName.equals(parameter.getName())) {
                return this.getImplementation(parameter);
            }
            continue;
        }
        return null;
    }
    
    private ManagedServiceAssertion.ImplementationRecord getImplementation(PolicyAssertion rootParameter) {
        String className = rootParameter.getAttributeValue(ManagedServiceAssertion.CLASS_NAME_ATTRIBUTE_QNAME);
        HashMap<QName, String> parameterMap = new HashMap();
        Iterator<PolicyAssertion> implementationParameters = rootParameter.getParametersIterator();
        Collection<ManagedServiceAssertion.NestedParameters> nestedParameters = new LinkedList();
        while (implementationParameters.hasNext()) {
            PolicyAssertion parameterAssertion = (PolicyAssertion)implementationParameters.next();
            QName parameterName = parameterAssertion.getName();
            if (parameterAssertion.hasParameters()) {
                Map<QName, String> nestedParameterMap = new HashMap();
                Iterator<PolicyAssertion> parameters = parameterAssertion.getParametersIterator();
                while (parameters.hasNext()) {
                    PolicyAssertion parameter = (PolicyAssertion)parameters.next();
                    String value2 = parameter.getValue();
                    if (value2 != null) {
                        value2 = value2.trim();
                    }
                    nestedParameterMap.put(parameter.getName(), value2);
                }
                nestedParameters.add(new ManagedServiceAssertion.NestedParameters(parameterName, nestedParameterMap));
            }
            else {
                String value = parameterAssertion.getValue();
                if (value != null) {
                    value = value.trim();
                }
                parameterMap.put(parameterName, value);
            }
        }
        return new ManagedServiceAssertion.ImplementationRecord(className, parameterMap, nestedParameters);
    }
    
    static {
        MANAGED_SERVICE_QNAME = new QName("http://java.sun.com/xml/ns/metro/management", "ManagedService");
        COMMUNICATION_SERVER_IMPLEMENTATIONS_PARAMETER_QNAME = new QName("http://java.sun.com/xml/ns/metro/management", "CommunicationServerImplementations");
        COMMUNICATION_SERVER_IMPLEMENTATION_PARAMETER_QNAME = new QName("http://java.sun.com/xml/ns/metro/management", "CommunicationServerImplementation");
        CONFIGURATOR_IMPLEMENTATION_PARAMETER_QNAME = new QName("http://java.sun.com/xml/ns/metro/management", "ConfiguratorImplementation");
        CONFIG_SAVER_IMPLEMENTATION_PARAMETER_QNAME = new QName("http://java.sun.com/xml/ns/metro/management", "ConfigSaverImplementation");
        CONFIG_READER_IMPLEMENTATION_PARAMETER_QNAME = new QName("http://java.sun.com/xml/ns/metro/management", "ConfigReaderImplementation");
        CLASS_NAME_ATTRIBUTE_QNAME = new QName("className");
        ENDPOINT_DISPOSE_DELAY_ATTRIBUTE_QNAME = new QName("endpointDisposeDelay");
        LOGGER = Logger.getLogger(ManagedServiceAssertion.class);
    }
    
    public static class ImplementationRecord
    {
        private final String implementation;
        private final Map<QName, String> parameters;
        private final Collection<ManagedServiceAssertion.NestedParameters> nestedParameters;
        
        protected ImplementationRecord(String implementation, Map<QName, String> parameters, Collection<ManagedServiceAssertion.NestedParameters> nestedParameters) {
            this.implementation = implementation;
            this.parameters = parameters;
            this.nestedParameters = nestedParameters;
        }
        
        public String getImplementation() {
            return this.implementation;
        }
        
        public Map<QName, String> getParameters() {
            return this.parameters;
        }
        
        public Collection<ManagedServiceAssertion.NestedParameters> getNestedParameters() {
            return this.nestedParameters;
        }
        
        @Override
        public boolean equals(Object obj) {
            if (obj == null) {
                return false;
            }
            if (this.getClass() != obj.getClass()) {
                return false;
            }
            ManagedServiceAssertion.ImplementationRecord other = (ManagedServiceAssertion.ImplementationRecord)obj;
            return ((this.implementation != null) ? this.implementation.equals(other.implementation) : (other.implementation == null)) && (this.parameters == other.parameters || (this.parameters != null && this.parameters.equals(other.parameters))) && (this.nestedParameters == other.nestedParameters || (this.nestedParameters != null && this.nestedParameters.equals(other.nestedParameters)));
        }
        
        @Override
        public int hashCode() {
            int hash = 3;
            hash = 53 * hash + ((this.implementation != null) ? this.implementation.hashCode() : 0);
            hash = 53 * hash + ((this.parameters != null) ? this.parameters.hashCode() : 0);
            hash = 53 * hash + ((this.nestedParameters != null) ? this.nestedParameters.hashCode() : 0);
            return hash;
        }
        
        @Override
        public String toString() {
            StringBuilder text = new StringBuilder("ImplementationRecord: ");
            text.append("implementation = \"").append(this.implementation).append("\", ");
            text.append("parameters = \"").append(this.parameters).append("\", ");
            text.append("nested parameters = \"").append(this.nestedParameters).append("\"");
            return text.toString();
        }
    }
    
    public static class NestedParameters
    {
        private final QName name;
        private final Map<QName, String> parameters;
        
        private NestedParameters(QName name, Map<QName, String> parameters) {
            this.name = name;
            this.parameters = parameters;
        }
        
        public QName getName() {
            return this.name;
        }
        
        public Map<QName, String> getParameters() {
            return this.parameters;
        }
        
        @Override
        public boolean equals(Object obj) {
            if (obj == null) {
                return false;
            }
            if (this.getClass() != obj.getClass()) {
                return false;
            }
            ManagedServiceAssertion.NestedParameters other = (ManagedServiceAssertion.NestedParameters)obj;
            return ((this.name != null) ? this.name.equals(other.name) : (other.name == null)) && (this.parameters == other.parameters || (this.parameters != null && this.parameters.equals(other.parameters)));
        }
        
        @Override
        public int hashCode() {
            int hash = 5;
            hash = 59 * hash + ((this.name != null) ? this.name.hashCode() : 0);
            hash = 59 * hash + ((this.parameters != null) ? this.parameters.hashCode() : 0);
            return hash;
        }
        
        @Override
        public String toString() {
            StringBuilder text = new StringBuilder("NestedParameters: ");
            text.append("name = \"").append(this.name).append("\", ");
            text.append("parameters = \"").append(this.parameters).append("\"");
            return text.toString();
        }
    }
}
